/*****************************************************************************
* Microchip Graphics Library
* Graphics Display Designer (GDD) Template
*****************************************************************************
* Dependencies:    See INCLUDES section below
* Processor:       PIC24F, PIC24H, dsPIC, PIC32
* Compiler:       	MPLAB C30 V3.22, MPLAB C32 V1.05b
* Linker:          MPLAB LINK30, MPLAB LINK32
* Company:         Microchip Technology Incorporated
*
* Software License Agreement
*
* Copyright (c) 2010 Microchip Technology Inc.  All rights reserved.
* Microchip licenses to you the right to use, modify, copy and distribute
* Software only when embedded on a Microchip microcontroller or digital
* signal controller, which is integrated into your product or third party
* product (pursuant to the sublicense terms in the accompanying license
* agreement).  
*
* You should refer to the license agreement accompanying this Software
* for additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY
* OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR
* PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR
* OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION,
* BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT
* DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL,
* INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
* COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY
* CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
* OR OTHER SIMILAR COSTS.
*
* Author               Date        Comment
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*
*****************************************************************************/
#include "Graphics\Graphics.h"
#include "GDD_Screens.h"

// Strings
const XCHAR mainScrn_BTN3text[] = "#3";
const XCHAR mainScrn_BTN4text[] = "#4";
const XCHAR mainScrn_BTN5text[] = "#5";
const XCHAR mainScrn_BTN6text[] = "#6";
const XCHAR mainScrn_BTN7text[] = "#7";
const XCHAR mainScrn_BTN8text[] = "#8";
const XCHAR mainScrn_BTN1text[] = "#1";
const XCHAR mainScrn_BTN2text[] = "#2";
const XCHAR mainScrn_OBJ_STATICTEXT_3_1text[] = "***LED CONTROL SCREEN***\nby Jose A Valerio, Jr.";
GOL_SCHEME* btnscheme;
GOL_SCHEME* txtscheme;
void (*CreateFunctionArray[NUM_GDD_SCREENS])();
void (*CreatePrimitivesFunctionArray[NUM_GDD_SCREENS])();
WORD currentGDDDemoScreenIndex;
static BYTE update = 0;
static BYTE updateGPL = 0;
//  Creates the First Screen
void GDDDemoCreateFirstScreen(void)
{
	currentGDDDemoScreenIndex = 0;
	update = 1;
	(*CreateFunctionArray[currentGDDDemoScreenIndex])();
}
//  Creates the Next Screen
void GDDDemoNextScreen(void)
{
	currentGDDDemoScreenIndex++;
	if(currentGDDDemoScreenIndex >= NUM_GDD_SCREENS)
	{
		currentGDDDemoScreenIndex = 0;
	}
	update = 1;
}
//  GOL Message Callback
void GDDDemoGOLMsgCallback(WORD objMsg, OBJ_HEADER* pObj, GOL_MSG* pMsg)
{
	//Future event handling support by GDD
}
//  GOL Draw Callback
void GDDDemoGOLDrawCallback(void)
{
	if(updateGPL)
	{
		(*CreatePrimitivesFunctionArray[currentGDDDemoScreenIndex])();
		updateGPL = 0;
	}
if(update)
	{
		(*CreateFunctionArray[currentGDDDemoScreenIndex])();
		if(CreatePrimitivesFunctionArray[currentGDDDemoScreenIndex] != NULL)
		{
			updateGPL = 1;
		}
		update = 0;
	}
}
//  Create Error Flash (Blue Screen)
void CreateError(char* string)
{
	// Blue Screen Error
	SetColor(119);
	ClearDevice();
	SetColor(-1);
// Flash Error Message
	if(string == NULL)
		{OutTextXY(0, 0, "Runtime Error.");}
	else{OutTextXY(0,0, string);}
}

void CreatemainScrn(void)
{
	GOLFree();
	SetColor(8);
//	ClearDevice();
	if(btnscheme != NULL) free(btnscheme);
	btnscheme = GOLCreateScheme();
	btnscheme->Color0 = 1048;
	btnscheme->Color1 = 65504;
	btnscheme->ColorDisabled = 34800;
	btnscheme->CommonBkColor = 55166;
	btnscheme->EmbossDkColor = 34815;
	btnscheme->EmbossLtColor = 2024;
	btnscheme->TextColor0 = 65535;
	btnscheme->TextColor1 = 63488;
	btnscheme->pFont = (void*)&gentium_8_200_0_0_0_0;
	if(txtscheme != NULL) free(txtscheme);
	txtscheme = GOLCreateScheme();
	txtscheme->Color0 = 31;
	txtscheme->Color1 = 63488;
	txtscheme->ColorDisabled = 55070;
	txtscheme->CommonBkColor = 0;
	txtscheme->EmbossDkColor = 0;
	txtscheme->EmbossLtColor = 0;
	txtscheme->TextColor0 = 65535;
	txtscheme->TextColor1 = 31;
	txtscheme->pFont = (void*)&gentium_8_200_0_0_0_0;
/*	BUTTON *pmainScrn_BTN3;
	pmainScrn_BTN3 = BtnCreate(mainScrn_BTN3,200,90,240,130,20,16384,NULL,(XCHAR*)mainScrn_BTN3text,btnscheme);
	if(pmainScrn_BTN3==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	BUTTON *pmainScrn_BTN4;
	pmainScrn_BTN4 = BtnCreate(mainScrn_BTN4,260,90,300,130,20,16384,NULL,(XCHAR*)mainScrn_BTN4text,btnscheme);
	if(pmainScrn_BTN4==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	BUTTON *pmainScrn_BTN5;
	pmainScrn_BTN5 = BtnCreate(mainScrn_BTN5,20,160,60,200,20,16384,NULL,(XCHAR*)mainScrn_BTN5text,btnscheme);
	if(pmainScrn_BTN5==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	BUTTON *pmainScrn_BTN6;
	pmainScrn_BTN6 = BtnCreate(mainScrn_BTN6,80,160,120,200,20,16384,NULL,(XCHAR*)mainScrn_BTN6text,btnscheme);
	if(pmainScrn_BTN6==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	BUTTON *pmainScrn_BTN7;
	pmainScrn_BTN7 = BtnCreate(mainScrn_BTN7,200,160,240,200,20,16384,NULL,(XCHAR*)mainScrn_BTN7text,btnscheme);
	if(pmainScrn_BTN7==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	BUTTON *pmainScrn_BTN8;
	pmainScrn_BTN8 = BtnCreate(mainScrn_BTN8,260,160,300,200,20,16384,NULL,(XCHAR*)mainScrn_BTN8text,btnscheme);
	if(pmainScrn_BTN8==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size 
	BUTTON *pmainScrn_BTN2;
	pmainScrn_BTN2 = BtnCreate(mainScrn_BTN2,79,90,119,130,20,16384,NULL,(XCHAR*)mainScrn_BTN2text,btnscheme);
	if(pmainScrn_BTN2==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size */
	BUTTON *pmainScrn_BTN1;
	pmainScrn_BTN1 = BtnCreate(mainScrn_BTN1,20,90,60,130,20,16384,NULL,(XCHAR*)mainScrn_BTN1text,btnscheme);
	if(pmainScrn_BTN1==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
	STATICTEXT *pmainScrn_OBJ_STATICTEXT_3_1;
	pmainScrn_OBJ_STATICTEXT_3_1 = StCreate(mainScrn_OBJ_STATICTEXT_3_1,20,24,300,62,16408,(XCHAR*)mainScrn_OBJ_STATICTEXT_3_1text,txtscheme);
	if(pmainScrn_OBJ_STATICTEXT_3_1==NULL){CreateError(0);while(1);} //Fatal Error, Check for memory leak or heap size
}

void (*CreateFunctionArray[NUM_GDD_SCREENS])(void)=
{
&CreatemainScrn
};

void (*CreatePrimitivesFunctionArray[NUM_GDD_SCREENS])(void)=
{
NULL
};
