/*********************************************************************
 *
 *	Hardware specific definitions
 *
 *********************************************************************
 * FileName:        HardwareProfile.h
 * Dependencies:    None
 * Processor:       PIC24F, PIC24H, PIC32MX
 * Compiler:        Microchip C32 v1.00 or higher
 *					Microchip C30 v3.01 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright ?2002-2008 Microchip Technology Inc.  All rights 
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and 
 * distribute: 
 * (i)  the Software when embedded on a Microchip microcontroller or 
 *      digital signal controller product (Device? which is 
 *      integrated into Licensees product; or
 * (ii) ONLY the Software driver source files ENC28J60.c and 
 *      ENC28J60.h ported to a non-Microchip device used in 
 *      conjunction with a Microchip ethernet controller for the 
 *      sole purpose of interfacing with the ethernet controller. 
 *
 * You should refer to the license agreement accompanying this 
 * Software for additional information regarding your rights and 
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS?WITHOUT 
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT 
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL 
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR 
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF 
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS 
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE 
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER 
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT 
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date		Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Howard Schlunder		10/03/06	Original, copied from Compiler.h
 ********************************************************************/

/*
 ********************************************************************
 * Hardware profile for a third party LCD module added for the hardware
 * 3.5" QVGA 16.7m color TFT LCD Module Evaluation Kit
 * Item number LVC75Z779 Eval Board (Version 1E)
 * stack on PIC24 Eval Board C2
 *
 * Author: John Leung
 * File: HardwareProfile.h
 * Company: TechToys Company
 * Web presence: www.TechToys.com.hk
 * Date: 15th March 2009
 *
 * Versions:
 * (1) Add GRAPHICS_PICTAIL_VERSION == GRAPHICS_THIRD_PARTY for 
 * 		TechToy's hardware LVC75Z779 Eval Board (Version 1E)
 * (2) __PIC24FJ256GB110__  not supported under the EEPROM section
 * (3) Change CS pin for EEPROM to RG13 for PIC24-eval-board C2 board
 * (4) Remove ADC_TEMP
 * (5) Change ADC_POT to AN0
 * (6) Add GRAPHICS_THIRD_PARTY for ADC IOS for Touch Pabel ADC
 *
 ********************************************************************
*/

#ifndef __HARDWARE_PROFILE_H
#define __HARDWARE_PROFILE_H

/*********************************************************************
* GetSystemClock() returns system clock frequency.
*
* GetPeripheralClock() returns peripheral clock frequency.
*
* GetInstructionClock() returns instruction clock frequency.
*
********************************************************************/
#include "GraphicsConfig.h"

#if defined(__PIC24F__)	|| defined(__PIC24H__)
    // PIC24 processor
#define	GetSystemClock()            (8000000ul)         // 8 MHz
#define	GetPeripheralClock()		(GetSystemClock()/2)
#define	GetInstructionClock()		(GetSystemClock()/2)

#elif defined(__PIC32MX__)
	// PIC32MX processor
#define	GetSystemClock()            (72000000ul)         // 8 MHz/2 x 18 PLL
#define	GetPeripheralClock()		(GetSystemClock()/(1<<OSCCONbits.PBDIV))
#define	GetInstructionClock()		(GetSystemClock()/PFMWSbits.CHECON)
#endif


/*********************************************************************
* IOS FOR THE DISPLAY CONTROLLER
*********************************************************************/
#if (GRAPHICS_PICTAIL_VERSION == 1)

// Definitions for reset pin
#define RST_TRIS_BIT       TRISCbits.TRISC1
#define RST_LAT_BIT        LATCbits.LATC1

// Definitions for RS pin
#define RS_TRIS_BIT        TRISBbits.TRISB15
#define RS_LAT_BIT         LATBbits.LATB15

// Definitions for CS pin
#define CS_TRIS_BIT        TRISDbits.TRISD8
#define CS_LAT_BIT         LATDbits.LATD8

// Definitions for FLASH CS pin 
#define CS_FLASH_LAT_BIT   LATDbits.LATD9
#define CS_FLASH_TRIS_BIT  TRISDbits.TRISD9


#elif (GRAPHICS_PICTAIL_VERSION == 2)

#if  (DISPLAY_CONTROLLER == LGDP4531)

// Definitions for reset pin
#define RST_TRIS_BIT       TRISCbits.TRISC1
#define RST_LAT_BIT        LATCbits.LATC1

// Definitions for RS pin
#define RS_TRIS_BIT        TRISCbits.TRISC2
#define RS_LAT_BIT         LATCbits.LATC2

// Definitions for CS pin
#define CS_TRIS_BIT        TRISDbits.TRISD10
#define CS_LAT_BIT         LATDbits.LATD10

// Definitions for FLASH CS pin 
#define CS_FLASH_LAT_BIT   LATDbits.LATD1
#define CS_FLASH_TRIS_BIT  TRISDbits.TRISD1

// Definitions for POWER ON pin
#define POWERON_LAT_BIT    LATCbits.LATC3
#define POWERON_TRIS_BIT   TRISCbits.TRISC3

#elif (DISPLAY_CONTROLLER == SSD1906)

// Definitions for reset line
#define RST_TRIS_BIT       TRISCbits.TRISC1
#define RST_LAT_BIT        LATCbits.LATC1

// Definitions for RS line
#define RS_TRIS_BIT        TRISCbits.TRISC2
#define RS_LAT_BIT         LATCbits.LATC2

// Definitions for CS line
#define CS_TRIS_BIT        TRISDbits.TRISD10
#define CS_LAT_BIT         LATDbits.LATD10

// Definitions for A0 line
#define A0_LAT_BIT         LATDbits.LATD3
#define A0_TRIS_BIT        TRISDbits.TRISD3

// Definitions for A17 line 
#define A17_LAT_BIT        LATGbits.LATG14
#define A17_TRIS_BIT       TRISGbits.TRISG14

#else

#error	GRAPHICS CONTROLLER IS NOT SUPPORTED

#endif  // (DISPLAY_CONTROLLER == ...

#elif (GRAPHICS_PICTAIL_VERSION == 3)

// Definitions for reset line
#define RST_TRIS_BIT       TRISCbits.TRISC1
#define RST_LAT_BIT        LATCbits.LATC1

// Definitions for RS line
#define RS_TRIS_BIT        TRISCbits.TRISC2
#define RS_LAT_BIT         LATCbits.LATC2

// Definitions for CS line
#define CS_TRIS_BIT        TRISDbits.TRISD10
#define CS_LAT_BIT         LATDbits.LATD10

#elif (GRAPHICS_PICTAIL_VERSION == GRAPHICS_THIRD_PARTY)
/*
*****************************************************************************
* Use 8080 system interface. PMP for PIC32MX series, otherwise, discrete
* IO method would be used for PIC24F series
*
* Function		mcu pins		LCD pins
* ==========	========		========
* SLEEP			RG15			SLEEP
* DATA LINES	PMD[7:0]		DAT[7:0] in 8-bit mode
* RESET			RA7				/RESET
* CHIP SELECT	RD11			/CS
* COMMAND/DATA	RA6				RS 
* WR STROBE		RD4/PMPWR		/WR
* RD STROBE		RD5/PMPRD		/RD
* LED backlight RF8				EN
*
*****************************************************************************
*/
//Definitions for SLEEP pin of SSD1928
#define SLEEP_TRIS_BIT	   TRISGbits.TRISG15
#define SLEEP_LAT_BIT	   LATGbits.LATG15

// Definitions for reset pin
#define RST_TRIS_BIT       TRISAbits.TRISA7
#define RST_LAT_BIT        LATAbits.LATA7

//Definition for RS pin
#define RS_TRIS_BIT			TRISAbits.TRISA6
#define RS_LAT_BIT			LATAbits.LATA6

// Definitions for CS pin
#define CS_TRIS_BIT        TRISDbits.TRISD11
#define CS_LAT_BIT         LATDbits.LATD11

// Definition for WR pin
#define WR_TRIS_BIT		   TRISDbits.TRISD4
#define WR_LAT_BIT		   LATDbits.LATD4

// Definition for RD pin
#define RD_TRIS_BIT		   TRISDbits.TRISD5
#define RD_LAT_BIT		   LATDbits.LATD5

//Defintion for backlight control
#define LED_TRIS_BIT		TRISFbits.TRISF8
#define LED_LAT_BIT			LATFbits.LATF8

#endif // (GRAPHICS_PICTAIL_VERSION == ...

/*********************************************************************
* IO FOR THE BEEPER PWM
*********************************************************************/

#define OC_TRIS_BIT         TRISDbits.TRISD0
#define OC_LAT_BIT          LATDbits.LATD0

/*********************************************************************
* IOS FOR THE EEPROM SPI
*********************************************************************/
#if defined (__C30__)
    #if defined( __PIC24FJ256GB110__ )
        // This PIM has RD12 rerouted to RA15
        //#define EEPROM_CS_TRIS       TRISGbits.TRISG0
        //#define EEPROM_CS_PORT       PORTGbits.RG0
    #else
        #define EEPROM_CS_TRIS       TRISGbits.TRISG13
        #define EEPROM_CS_PORT       PORTGbits.RG13
    #endif
#elif defined( __PIC32MX__ )
    #define EEPROM_CS_TRIS       TRISGbits.TRISG13
    #define EEPROM_CS_PORT       PORTGbits.RG13
#endif
#define EEPROM_SCK_TRIS      TRISGbits.TRISG6
#define EEPROM_SDO_TRIS      TRISGbits.TRISG8
#define EEPROM_SDI_TRIS      TRISGbits.TRISG7

/*********************************************************************
* IOS FOR THE TOUCH SCREEN
*********************************************************************/
// ADC channel constants
#ifdef __PIC32MX__
    //#define ADC_TEMP        ADC_CH0_POS_SAMPLEA_AN4
    #define ADC_POT         ADC_CH0_POS_SAMPLEA_AN0
#else
    //#define ADC_TEMP        4
    #define ADC_POT         0
#endif

#if (GRAPHICS_PICTAIL_VERSION == 1)

    #ifdef __PIC32MX__
        #define ADC_XPOS        ADC_CH0_POS_SAMPLEA_AN13
        #define ADC_YPOS        ADC_CH0_POS_SAMPLEA_AN12
    #else
        #define ADC_XPOS        13
        #define ADC_YPOS        12
    #endif
    
    // Y port definitions
    #define ADPCFG_XPOS     AD1PCFGbits.PCFG13
    #define LAT_XPOS        LATBbits.LATB13 
    #define LAT_XNEG        LATBbits.LATB11
    #define TRIS_XPOS       TRISBbits.TRISB13
    #define TRIS_XNEG       TRISBbits.TRISB11 
    
    // X port definitions
    #define ADPCFG_YPOS     AD1PCFGbits.PCFG12
    #define LAT_YPOS        LATBbits.LATB12  
    #define LAT_YNEG        LATBbits.LATB10 
    #define TRIS_YPOS       TRISBbits.TRISB12
    #define TRIS_YNEG       TRISBbits.TRISB10 

#elif (GRAPHICS_PICTAIL_VERSION == 2)

    #ifdef __PIC32MX__
        #define ADC_XPOS        ADC_CH0_POS_SAMPLEA_AN11
        #define ADC_YPOS        ADC_CH0_POS_SAMPLEA_AN10
    #else
        #define ADC_XPOS        11
        #define ADC_YPOS        10
    #endif
    
    // Y port definitions
    #define ADPCFG_XPOS     AD1PCFGbits.PCFG11
    #define LAT_XPOS        LATBbits.LATB11
    #define LAT_XNEG        LATGbits.LATG13
    #define TRIS_XPOS       TRISBbits.TRISB11
    #define TRIS_XNEG       TRISGbits.TRISG13
    
    // X port definitions
    #define ADPCFG_YPOS     AD1PCFGbits.PCFG10
    #define LAT_YPOS        LATBbits.LATB10  
    #define LAT_YNEG        LATGbits.LATG12 
    #define TRIS_YPOS       TRISBbits.TRISB10
    #define TRIS_YNEG       TRISGbits.TRISG12 

#elif (GRAPHICS_PICTAIL_VERSION == 3)

    #ifdef __PIC32MX__
        #define ADC_XPOS        ADC_CH0_POS_SAMPLEA_AN11
        #define ADC_YPOS        ADC_CH0_POS_SAMPLEA_AN10
    #else
        #define ADC_XPOS        11
        #define ADC_YPOS        10
    #endif
    
    // Y port definitions
    #define ADPCFG_XPOS     AD1PCFGbits.PCFG11
    #define LAT_XPOS        LATBbits.LATB11
    #define LAT_XNEG        LATDbits.LATD9
    #define TRIS_XPOS       TRISBbits.TRISB11
    #define TRIS_XNEG       TRISDbits.TRISD9
    
    // X port definitions
    #define ADPCFG_YPOS     AD1PCFGbits.PCFG10
    #define LAT_YPOS        LATBbits.LATB10  
    #define LAT_YNEG        LATDbits.LATD8 
    #define TRIS_YPOS       TRISBbits.TRISB10
    #define TRIS_YNEG       TRISDbits.TRISD8 

#elif (GRAPHICS_PICTAIL_VERSION == GRAPHICS_THIRD_PARTY)

    #ifdef __PIC32MX__
        #define ADC_XPOS        ADC_CH0_POS_SAMPLEA_AN10
        #define ADC_YPOS        ADC_CH0_POS_SAMPLEA_AN13
    #else
        #define ADC_XPOS        10
        #define ADC_YPOS        13
    #endif
    
    // Y port definitions
    #define ADPCFG_XPOS     AD1PCFGbits.PCFG10
    #define LAT_XPOS        LATBbits.LATB10
    #define LAT_XNEG        LATBbits.LATB12
    #define TRIS_XPOS       TRISBbits.TRISB10
    #define TRIS_XNEG       TRISBbits.TRISB12
    
    // X port definitions
    #define ADPCFG_YPOS     AD1PCFGbits.PCFG13
    #define LAT_YPOS        LATBbits.LATB13
    #define LAT_YNEG        LATBbits.LATB11 
    #define TRIS_YPOS       TRISBbits.TRISB13
    #define TRIS_YNEG       TRISBbits.TRISB11 

#endif

#endif // __HARDWARE_PROFILE_H
